//
//  Brahe
//  A heterogenous library of mathematical and scientific functions
//
//  prng.h
//
//  Copyright 2006, 2007 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//  Closed-source licenses are available from the author at reasonable
//  prices.
//

#if !defined(LIBBRAHE_PRNG_H)
#define LIBBRAHE_PRNG_H

#include "mathtools.h"

#if defined(__cplusplus)
extern "C" {
#endif

// define types of prngs
typedef enum
{
    BRAHE_PRNG_MARSENNE_TWISTER = 0,
    BRAHE_PRNG_KISS,
    BRAHE_PRNG_CMWC4096,
    BRAHE_PRNG_MWC1038,
    BRAHE_PRNG_ISAAC
} brahe_prng_type_t;

typedef struct
{
    brahe_prng_type_t m_type;  // the algorithm we're using
    uint32_t          m_seed;  // "seed" that defines the sequence
    void *            m_data1; // array or table
    void *	          m_data2; // additional array or table
    size_t            m_i;     // only used by some algorithms
    size_t            m_a;     // only used by some algorithms
    size_t            m_b;     // only used by some algorithms
    size_t            m_c;     // only used by some algorithms
} brahe_prng_state_t;

static uint32_t BRAHE_UNKNOWN_SEED = 0;

//! Initialize a psuedo-random number generator (PRNG)
/*!
    Initializes a psuedo-random number generator using a specified algorithm.
*/
bool brahe_prng_init(brahe_prng_state_t * prng_state, brahe_prng_type_t type, uint32_t seed);

//! Free resources used by PRNG
/*!
    Frees the resources used by a PRNG
*/
void brahe_prng_free(brahe_prng_state_t * prng_state);

//!  Get the next integer
/*!
    Returns the next uint32_t in sequence.
    \return A pseudorandom uint32_t value
*/
uint32_t brahe_prng_next(brahe_prng_state_t * prng_state);

//! Get the next integer in the range [lo,hi]
/*!
    Returns the next int_value between lo and hi, inclusive.
    \param lo - Minimum value of result
    \param hi - Maximum value of result
    \return A pseudorandom uint32_t value
*/
uint32_t brahe_prng_range(brahe_prng_state_t * prng_state, uint32_t lo, uint32_t hi);

//! Get the next random value as a size_t index
/*!
    Returns the next value as a size_t "index" in the range [0,length).
    \param length - Maximum value of result
    \return A pseudorandom size_t value
*/
size_t brahe_prng_index(brahe_prng_state_t * prng_state, size_t length);

//! Get the next number in the range [0,1]
/*!
    Returns the next real number in the range [0,1], i.e., a number
    greater than or equal to 0 and less than or equal to 1.
    Provides 32-bit precision.
    \return A pseudorandom double value
*/
double brahe_prng_real1(brahe_prng_state_t * prng_state);

//! Get the next number in the range [0,1)
/*!
    Returns the next real number in the range [0,1), i.e., a number
    greater than or equal to 0 and less than 1.
    Provides 32-bit precision.
    \return A pseudorandom double value
*/
double brahe_prng_real2(brahe_prng_state_t * prng_state);

//! Get the next number in the range (0,1)
/*!
    Returns the next real number in the range (0,1), i.e., a number
    greater than 0 and less than 1.
    Provides 32-bit precision.
    \return A pseudorandom double value
*/
double brahe_prng_real3(brahe_prng_state_t * prng_state);

//! Get the next number in the range [0,1)
/*!
    Returns the next real number in the range [0,1), i.e., a number
    greater than or equal to 0 and less than 1.
    Provides 53-bit precision.
    \return A pseudorandom double value
*/
double brahe_prng_real53(brahe_prng_state_t * prng_state);

#if defined(__cplusplus)
}
#endif

#endif
